package com.gzlh.bus;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONUtil;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.device.capture.factory.CaptureFactory;
import com.gzlh.device.capture.properties.CapturePropertiesConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.SystemObject;
import com.gzlh.device.electron.factory.ElectronFactory;
import com.gzlh.device.infrared.config.RedPropertiesConfig;
import com.gzlh.device.infrared.factory.RedFactory;
import com.gzlh.device.led.factory.LedFactory;
import com.gzlh.device.plc.event.PLCEvent;
import com.gzlh.device.plc.handler.PLCHadnler;
import com.gzlh.device.weighbridge.config.WeighbridgePropertiesConfig;
import com.gzlh.device.weighbridge.factory.WeighbridgeFactory;
import com.gzlh.utils.DeviceCache;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
public class EventThread implements Runnable{
    private String eventName;

    public EventThread(String eventName) {
        this.eventName=eventName;
    }

    /**
     * 处理事件
     *
     * @param eventName 服务.事件名 比如led.ready
     */
    public void handlerEvent(String eventName) {
        SerialSetting serialSetting=EventConfig.serialSetting;
        LedFactory ledFactory= SystemObject.ledFactory;
        RedFactory redFactory=SystemObject.redFactory;
        WeighbridgeFactory weighbridgeFactory=SystemObject.weighbridgeFactory;
        ElectronFactory electronFactory= SystemObject.electronFactory;
        WeighbridgePropertiesConfig weighbridgePropertiesConfig=SystemObject.weighbridgePropertiesConfig;
        RedPropertiesConfig redPropertiesConfig=SystemObject.redPropertiesConfig;
        PLCHadnler plcHadnler= SpringUtil.getBean(PLCHadnler.class);
        CaptureFactory captureFactory=SystemObject.captureFactory;
        CapturePropertiesConfig capturePropertiesConfig=SystemObject.capturePropertiesConfig;
        log.info("event:{}", eventName);
        EventConfig.eventDTOList.stream().filter(eventDTO -> StrUtil.equals(eventName, eventDTO.getName()))
                .findFirst().ifPresent(eventDTO -> {
            List<String> actionList = eventDTO.getActionList().getAction();
            //依次执行动作
            actionList.forEach(action -> {
                String module = StrUtil.subBefore(action, ".", true);
                String command = StrUtil.subAfter(action, ".", true);
//                中断事件动作
                if (DeviceCache.isInterrupt()){
                    DeviceCache.setInterrupt(false);
                    return;
                }
                if (StrUtil.equals(module, ModuleEnum.INFRARED_MODULE.getModuleEn())) {
                    //动作属于红外
                    redFactory.handler(redPropertiesConfig.getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.LED_MODULE.getModuleEn())) {
                    //动作属于led
                    ledFactory.handler(serialSetting.getLed().getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.WEIGHBRIDGE_MODULE.getModuleEn())) {
                    //动作属于地磅
                    weighbridgeFactory.handler(weighbridgePropertiesConfig.getBrand()).handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.CAPTURE_MODULE.getModuleEn())) {
                    //动作属于抓拍单元--车牌识别
                    captureFactory.handler(capturePropertiesConfig.getBrand()).handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.PLC_MODULE.getModuleEn())) {
                    //动作属于plc
                    plcHadnler.handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.CAR_MODULE.getModuleEn())) {
                    //动作属于电子车牌
                    electronFactory.handler(serialSetting.getCar().getBrand()).handlerAction(command);
                }   else if (StrUtil.equals(module, ModuleEnum.CENTER_MODULE.getModuleEn())) {
                    //动作属于中心总线
                    handlerAction(command);
                }
                ThreadUtil.sleep(250);
            });
        });

    }

    /**
     * 调度中心命令处理
     *
     * @param action
     */
    private void handlerAction(String action) {
        if (StrUtil.contains(action, CenterActionCommand.SLEEP_COMMAND)) {
            //休眠
            handlerSleep(action);
        }else if (StrUtil.contains(action, CenterActionCommand.SUBMIT)){
            handlerSubmit();
        }
    }

    /**
     * 调度中心休眠
     *
     * @param action
     */
    private void handlerSleep(String action) {
        String time = StrUtil.subBefore(StrUtil.subAfter(action, "(", true), ")", true);
        if (NumberUtil.isInteger(time)) {
            ThreadUtil.sleep(Long.parseLong(time));
        }
    }

    /**
     * 提交数据
     *
     * @param
     */
    private void handlerSubmit() {
        String address = SystemObject.applicationConfig.getServerUrl();
        String api="/open/submit";
        HttpResponse response = request(address+api, JSONUtil.toJsonStr(EventDataManager.getCacheData()));
        EventBus eventBus= SpringUtil.getBean(EventBus.class);
        if (response!=null&&response.getStatus()==200){
            eventBus.startEvent(ModuleEnum.PLC_MODULE.getModuleEn()+"."+ PLCEvent.RAILING_RISE);
        }else {
            eventBus.startEvent(ModuleEnum.PLC_MODULE.getModuleEn()+"."+ PLCEvent.CHECK_FAILED);
        }
//        重置记录的设备触发时间
        DeviceCache.resetTimeMap();
    }

    public HttpResponse request(String url,String json) {

        HttpResponse res = null;
//        url = "http://192.168.1.8:9191/open/submit";
        long startTime = System.currentTimeMillis();
        System.out.println(startTime);

        String body = json;
        try {
//            System.out.printf("请求地址：%s, 请求体：%s ", url,body);

            res = HttpRequest.post(url)
                    .header("Content-Type", "application/json;character=utf-8;")
                    .body(body)
                    .execute();
            String bodyResult = res.body();
//            System.out.printf("ReqStatus:%s,返回结果：{%s} ",res.getStatus(), bodyResult);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return res;
    }



    @Override
    public void run(){
        handlerEvent(eventName);
    }
}
