package com.gzlh.device.capture.utils;

import cn.hutool.log.StaticLog;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

public class ZKTecoUtils {

    public static boolean sendCmd(Socket socket, String cmd) {
        try {
            int len = cmd.getBytes().length;
            byte[] header = {'V', 'Z', 0, 0, 0, 0, 0, 0};
            header[4] += (byte) ((len >> 24) & 0xFF);
            header[5] += (byte) ((len >> 16) & 0xFF);
            header[6] += (byte) ((len >> 8) & 0xFF);
            header[7] += (byte) (len & 0xFF);
            OutputStream out = socket.getOutputStream();
            out.write(header);
            out.write(cmd.getBytes());
        } catch (Exception e) {
            StaticLog.error("熵基下发道闸命令异常:{},{}", cmd, e.getMessage());
            return false;
        }
        return true;
    }

    public static String sendAndGetResult(Socket socket, String cmd) {
        boolean success = sendCmd(socket, cmd);
        if (!success) {
            return "";
        }
        int sn_len = recvPacketSize(socket);
        if (sn_len > 0) {
            //接收实际数据
            byte[] data = new byte[sn_len];
            int recvLen = ZKTecoUtils.recvBlock(socket, data, sn_len);
            String result = new String(data, 0, recvLen);
            StaticLog.info("熵基下发命令返回:{},{}", cmd, result);
            return result;
        }
        return "";
    }

    public static int recvPacketSize(Socket socket) {
        byte[] header = new byte[8];
        int recvLen = recvBlock(socket, header, 8);
        if (recvLen <= 0) {
            return -1;
        }

        if (header[0] != 'V' || header[1] != 'Z') {
            //格式不对
            return -1;
        }

        if (header[2] == 1) {
            //心跳包
            return 0;
        }

        return convBytesToInt(header, 4);
    }

    private static int convBytesToInt(byte[] buff, int offset) {
        //4bytes 转为int，要考虑机器的大小端问题
        int len, byteValue;
        len = 0;
        byteValue = (0x000000FF & ((int) buff[offset]));
        len += byteValue << 24;
        byteValue = (0x000000FF & ((int) buff[offset + 1]));
        len += byteValue << 16;
        byteValue = (0x000000FF & ((int) buff[offset + 2]));
        len += byteValue << 8;
        byteValue = (0x000000FF & ((int) buff[offset + 3]));
        len += byteValue;
        return len;
    }

    //接收指定长度的数据，收完为止
    public static int recvBlock(Socket socket, byte[] buff, int len) {
        try {
            InputStream in = socket.getInputStream();
            int totleRecvLen = 0;
            int recvLen;
            while (totleRecvLen < len) {
                recvLen = in.read(buff, totleRecvLen, len - totleRecvLen);
                totleRecvLen += recvLen;
            }
            return len;
        } catch (Exception e) {
            // System.out.println("Error:"+e);
            return -1;
        }
    }

}
