package com.gzlh.device.led.utils;

import com.gzlh.config.SystemObject;
import com.gzlh.device.led.action.FengLiYuanCmdEnum;
import com.gzlh.utils.WordHandlerUtils;
import org.springframework.util.StringUtils;

import java.util.*;

public class FengLiYuanPackage {
    /**
     * 帧头
     */
    private static final String HEAD = "F501";

    private static final String LedAddr="01";

    private static final String send="00";


    public static String buildPackage(String command,String data) {
        //包数据= 起始字（两字节长度，固定0xF5,0x01）+长度（两字节长度）+屏地址+命令+发送+数据+总包校验
        //长度=2Byte，从[屏地址]到[数据]最后字节。高字节在前，低字节在后。
        //发送=0X00=发送，不需要应答，0X01=发送，需要应答
        //数据=N Byte，可以为空
        //总包校验=1byte，是从[起始字]到[数据]最后字节所有字节的异或校验

        String str=LedAddr+command+send+data;

        StringBuilder len = new StringBuilder(Integer.toHexString(str.length() / 2));
        if (len.length()>4){
            throw new RuntimeException("数据超出长度");
        }
        while (len.length()<4){
            len.insert(0, "0");
        }
        String dataStr = HEAD+len+str;

        return xorComputeResult(dataStr.toUpperCase());
    }

    //输入字符串进行异或校验，并加上校验结果
    public static String xorComputeResult(String data) {

        if (data.length()%2!=0){
            return null;
        }
        int xorCheckSum = 0;
        List<String> list= new ArrayList<>();
        for (int i=0;i<data.length();i+=2){
            list.add(data.substring(i,i+2));
        }

        for (int i = 0; i < list.size(); i++) {
            xorCheckSum^=Integer.valueOf(list.get(i),16);
        }
//        System.out.println(xorCheckSum);
        String hex = Integer.toHexString(xorCheckSum);
        if (hex.length() == 1) {
            hex = '0' + hex;
        }

        list.add(hex.toUpperCase(Locale.getDefault()));

        String str = list.toString().substring(1, list.toString().length() - 1);
        return str.replaceAll(", ", "");
    }

    public static String eternalMsg(String line,String showType, String date,String speed,String color,String data) {
//        line 行号01为第一行，02为第二行，03为第三行，04为第四行,05为第五行，06为第六行，07为第七行，08为第八行
//        showType 显示方式00 左移入 01 上移入 02 下移入 03 左展入 04 右展入 05 上展入 06 下展入 07 横向展开 08 横向闭合 09 纵向展开 0A 纵向闭合 0B 同时显示
//        date 日期显示方式 00 固定信息 01日期+固定信息 02 日期
//        speed 速度 00—05 备注：00最快，05最慢
//        color 颜色 01 红色 02 绿色 03 黄色 04 蓝色 05 粉色 06 青色 07 白色
        line=StringUtils.isEmpty(line)?"01":line;
        showType=StringUtils.isEmpty(showType)?"00":showType;
        date=StringUtils.isEmpty(date)?"00":date;
        speed=StringUtils.isEmpty(speed)?"02":speed;
        color=StringUtils.isEmpty(color)?"01":color;
        String ascii = WordHandlerUtils.msgToASCII(data);
        String len = Integer.toHexString(ascii.length() / 2).toUpperCase();
        if (len.length() == 1) {
            len = '0' + len;
        }
        return line+showType+date+speed+color+len+ascii;
    }

    public static String tempMsg(String line,String showType, String time,String speed,String color,String data) {
//        line 行号01为第一行，02为第二行，03为第三行，04为第四行,05为第五行，06为第六行，07为第七行，08为第八行
//        showType 显示方式00 左移入 01 上移入 02 下移入 03 左展入 04 右展入 05 上展入 06 下展入 07 横向展开 08 横向闭合 09 纵向展开 0A 纵向闭合 0B 同时显示
//        time 停留时间 00—ff 备注：00表示0秒，0秒处理为永久显示 05表示停留5秒,FF表示停留255秒 停留时间到后，LED显示临时信息自动切换到显示固定信息
//        speed 速度 00—05 备注：00最快，05最慢
//        color 颜色 01 红色 02 绿色 03 黄色 04 蓝色 05 粉色 06 青色 07 白色
        line=StringUtils.isEmpty(line)?"01":line;
        showType=StringUtils.isEmpty(showType)?"00":showType;
        time=StringUtils.isEmpty(time)?"00":time;
        speed=StringUtils.isEmpty(speed)?"02":speed;
        color=StringUtils.isEmpty(color)?"01":color;
        String ascii = WordHandlerUtils.msgToASCII(data);
        String len = Integer.toHexString(ascii.length() / 2).toUpperCase();
        if (len.length() == 1) {
            len = '0' + len;
        }
        return line+showType+time+speed+color+len+ascii;
    }


    public static void main(String[] args) {
//        String line1=tempMsg("01", "0B", "05", "00", "", "粤A.12345");
//        String line2=tempMsg("02", "00", "00", "00", "", "请交费10元");
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();


        String hour =""+ calendar.get(Calendar.HOUR_OF_DAY);
        String minute =""+calendar.get(Calendar.MINUTE);
        String second =""+calendar.get(Calendar.SECOND);
        if (hour.length()<2){
            hour="0"+hour;
        }
        if (minute.length()<2){
            minute="0"+minute;
        }
        if (second.length()<2){
            second="0"+second;
        }
        System.out.println(hour+","+minute+","+second);
        String line1=eternalMsg("01", "00", "01", "00", "", "欢迎光临");
        String line2=eternalMsg("02", "00", "00", "00", "", "深圳市[@2]丰利源[@3]有限公司");

        System.out.println(line1+","+line1.length());

        System.out.println(line2+","+line2.length());
        String s = buildPackage(FengLiYuanCmdEnum.ETERNAL.getCode(), line1 + line2);

//        String s = buildPackage(FengLiYuanCmdEnum.TEMP.getCode(), line1 + line2);
        System.out.println(s);
    }
}
