package com.gzlh.device.weighbridge.handler.impl;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.gzlh.bus.EventConfig;
import com.gzlh.bus.EventDataManager;
import com.gzlh.config.ApplicationConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.bus.EventBus;
import com.gzlh.config.SystemObject;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.device.led.factory.LedFactory;
import com.gzlh.device.led.utils.LedOptions;
import com.gzlh.device.weighbridge.config.WeighbridgePropertiesConfig;
import com.gzlh.device.weighbridge.event.WeighbridgeEvent;
import com.gzlh.utils.DeviceCache;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;

public class CommonWeighbridgeHandler {


    private final static List<Integer> WEIGHT_CACHE = new CopyOnWriteArrayList<>();


    public static boolean isStart = false;

    /**
     * 添加地磅数据到缓存
     *
     * @param weight 地磅数
     */
    public static void cacheWeight(int weight) {
        if (!isStart) {
            WEIGHT_CACHE.clear();
            return;
        }
        WEIGHT_CACHE.add(weight);
    }

    /**
     * 开始读取地磅数据
     */
    public static void startToRead() {
        isStart = true;
        ThreadUtil.execute(new WatchThread());
    }

    /**
     * 监控地磅缓存是否读取到了数据
     * 监控2秒内的地磅数
     */
    static class WatchThread implements Runnable {
        private int time = 3000;

        @Override
        public void run() {
            int averaWeight = 0;
            String deviceStatus= null;
            SerialSetting.PlcDTO.StatusDTO status = EventConfig.serialSetting.getPlc().getStatus();

            while (time > 0) {
                time = time - 1000;
                averaWeight = WEIGHT_CACHE.stream().collect(Collectors.averagingInt(Integer::intValue)).intValue();
                averaWeight = 12580;
                deviceStatus = new StringBuffer(DeviceCache.getPlcStatus()).reverse().toString();
                String fRed = String.valueOf(deviceStatus.charAt(Integer.parseInt(status.getFRedPoint())));
                String bRed = String.valueOf(deviceStatus.charAt(Integer.parseInt(status.getBRedPoint())));
                String radar = String.valueOf(deviceStatus.charAt(Integer.parseInt(status.getRadarPoint())));
                if (fRed.equals("1")||bRed.equals("1")||radar.equals("1")){
                    DeviceCache.setInterrupt(true);
                    LedOptions options = new LedOptions().setLine("04").setColor("01").setShowType("00");
                    SystemObject.ledFactory.handler(EventConfig.serialSetting.getLed().getBrand())
                            .sendMsg("车辆位置异常或异常跟车，请完全退出通道后再次进入通道",options);
                    break;
                }
                ThreadUtil.sleep(1000);
            }
//            EventBus eventBus = SpringUtil.getBean(EventBus.class);
//            WeighbridgePropertiesConfig weighbridgePropertiesConfig = SpringUtil.getBean(WeighbridgePropertiesConfig.class);
//            if (averaWeight == 0 || weighbridgePropertiesConfig.getMinKg() > averaWeight) {
//                //todo read timeout
//                //读取超时
//                eventBus.startEvent(ModuleEnum.WEIGHBRIDGE_MODULE.getModuleEn() + "." + WeighbridgeEvent.TIMEOUT);
//                return;
//            }
            isStart=false;
            ApplicationConfig applicationConfig = SpringUtil.getBean(ApplicationConfig.class);
            //上传地磅数据
            String channelName = applicationConfig.getChannelName();
            String serverUrl = applicationConfig.getServerUrl();
            EventDataManager.cacheData("weight",averaWeight);
        }
    }
}
