package com.gzlh.startup;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.XML;
import com.gzlh.background.client.BackgroundClientNetty;
import com.gzlh.background.properties.BackgroundPropertiesConfig;
import com.gzlh.bus.EventConfig;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.device.capture.properties.CapturePropertiesConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.SystemObject;
import com.gzlh.config.dto.ApplicationConfigDTO;
import com.gzlh.bus.EventBus;
import com.gzlh.config.hksdk.HCNetSDK;
import com.gzlh.config.hksdk.HkUtils;
import com.gzlh.config.hksdk.bo.HKCacheManager;
import com.gzlh.device.electron.client.ElectronNettyConfig;
import com.gzlh.device.led.client.LedNettyConfig;
import com.gzlh.device.infrared.client.RedNettyConfig;
import com.gzlh.device.plc.client.PlcNettyConfig;
import com.gzlh.device.weighbridge.client.WeighbridgeNettyConfig;
import com.gzlh.utils.WordHandlerUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.env.PropertyResolver;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.util.Properties;

@Component
@Slf4j
public class StartupRunner implements CommandLineRunner {

    @Resource
    private WeighbridgeNettyConfig weighbridgeNettyConfig;

    @Resource
    private LedNettyConfig ledNettyConfig;
    @Resource
    private PlcNettyConfig plcNettyConfig;
    @Resource
    private ElectronNettyConfig electronNettyConfig;


    @Resource
    private BackgroundPropertiesConfig backgroundPropertiesConfig;
    @Resource
    private BackgroundClientNetty backgroundClientNetty;

    @Resource
    private HkUtils hkUtils;
    @Resource
    private HCNetSDK hcNetSDK;
    @Override
    public void run(String... args) throws Exception {
        SerialSetting serialSetting=EventConfig.serialSetting;
        if (serialSetting.getWeighbridge().getEnable()) {
            ThreadUtil.execute(() -> weighbridgeNettyConfig.connect());
        }
        if (serialSetting.getLed().getEnable()) {
            ThreadUtil.execute(() -> ledNettyConfig.connect());
        }
        if (serialSetting.getPlc().getEnable()){
            ThreadUtil.execute(() -> plcNettyConfig.connect());
        }
        if (serialSetting.getElectron().getEnable()){
            ThreadUtil.execute(() -> electronNettyConfig.connect());
        }
        if (backgroundPropertiesConfig.isEnable()) {
            ThreadUtil.execute(() -> backgroundClientNetty.connect());
        }
        if (SystemObject.capturePropertiesConfig.isEnable()){
            CapturePropertiesConfig capturePropertiesConfig=SystemObject.capturePropertiesConfig;
            short port= (short) capturePropertiesConfig.getPort();
            String ip=capturePropertiesConfig.getHost();
            int userId = hkUtils.Login_V40(ip, port, capturePropertiesConfig.getUsername(), capturePropertiesConfig.getPwd(), hcNetSDK);
            if (userId!=-1){
                log.info("登录海康抓拍设备:{},{}",ip,userId);
                hkUtils.SetAlarm(hcNetSDK,userId);
                HKCacheManager.add(ModuleEnum.CAPTURE_MODULE,userId);
            }else {
                log.info("登录海康抓拍设备失败:{},{}",hcNetSDK.NET_DVR_GetLastError(),userId);
            }
        }

    }


}
